@@ -7,34 +7,32 @@ from deprecated import deprecated
 import numpy as np
 from clease.structgen import structure_generator
 
-MSG = 'Import {} from clease.structgen.structure_generator instead'
-DEP_VERSION = '0.10.2'  # Deprecation version
+MSG = "Import {} from clease.structgen.structure_generator instead"
+DEP_VERSION = "0.10.2"  # Deprecation version
 
-MODULE_DEP_MSG = ("""
-The clease.new_struct module has been moved as of version {}.
-Please use the clease.structgen.structure_generator module instead""".format(DEP_VERSION))
+MODULE_DEP_MSG = f"""
+The clease.new_struct module has been moved as of version {DEP_VERSION}.
+Please use the clease.structgen.structure_generator module instead"""
 
 # Print the message on import
 warn(MODULE_DEP_MSG, np.VisibleDeprecationWarning)
 
-# pylint: disable=too-few-public-methods
 
-
-@deprecated(version=DEP_VERSION, reason=MSG.format('StructureGenerator'))
+@deprecated(version=DEP_VERSION, reason=MSG.format("StructureGenerator"))
 class StructureGenerator(structure_generator.StructureGenerator):
     pass
 
 
-@deprecated(version=DEP_VERSION, reason=MSG.format('GSStructure'))
+@deprecated(version=DEP_VERSION, reason=MSG.format("GSStructure"))
 class GSStructure(structure_generator.GSStructure):
     pass
 
 
-@deprecated(version=DEP_VERSION, reason=MSG.format('MetropolisTrajectory'))
+@deprecated(version=DEP_VERSION, reason=MSG.format("MetropolisTrajectory"))
 class MetropolisTrajectory(structure_generator.MetropolisTrajectory):
     pass
 
 
-@deprecated(version=DEP_VERSION, reason=MSG.format('ProbeStructure'))
+@deprecated(version=DEP_VERSION, reason=MSG.format("ProbeStructure"))
 class ProbeStructure(structure_generator.ProbeStructure):
     pass
